@@ -510,9 +510,7 @@ End:\" construct).")
 			sub 
 			)) 
 		    mapped)))
-    (let ((result (cons regexp sub-list)))
-      (byte-compile 'result)
-      result)))
+    (cons regexp sub-list)))
 
 (defun forth-compile-words ()
   "Compile the the words from `forth-words' and `forth-indent-words' into
@@ -726,12 +724,11 @@ End:\" construct).")
 		 (get-text-property from 'fontified))
        (forth-update-properties from to)))))
 
-(eval-when-compile
-  (byte-compile 'forth-set-word-properties)
-  (byte-compile 'forth-next-known-forth-word)
-  (byte-compile 'forth-update-properties)
-  (byte-compile 'forth-delete-properties)
-  (byte-compile 'forth-get-regexp-branch)) 
+(byte-compile 'forth-set-word-properties)
+(byte-compile 'forth-next-known-forth-word)
+(byte-compile 'forth-update-properties)
+(byte-compile 'forth-delete-properties)
+(byte-compile 'forth-get-regexp-branch)
 
 ;;; imenu support
 ;;;
